@@ -297,41 +297,51 @@ unif()
 //                         attrs$data,
 //                         attrs$fpInfos,
 //                         attrs$ucInfos,
+//                         attrs$fixInfos,
 //                         attrs$distribution,
 //                         newdata,
 //                         options,
 //                         marginalz)
 
+
+
+
+// [[Rcpp::export]]
 SEXP
-cpp_sampleGlm(SEXP r_interface)
+cpp_sampleGlm(      List rcpp_model,List rcpp_data, List rcpp_fpInfos, List rcpp_ucInfos,
+                    List rcpp_fixInfos, List rcpp_distribution, List rcpp_searchConfig,
+                    List rcpp_options, List rcpp_marginalz)
 {
     // ----------------------------------------------------------------------------------
     // extract arguments
     // ----------------------------------------------------------------------------------
 
-    r_interface = CDR(r_interface);
-    List rcpp_model(CAR(r_interface));
-
-    r_interface = CDR(r_interface);
-    List rcpp_data(CAR(r_interface));
-
-    r_interface = CDR(r_interface);
-    List rcpp_fpInfos(CAR(r_interface));
-
-    r_interface = CDR(r_interface);
-    List rcpp_ucInfos(CAR(r_interface));
-
-    r_interface = CDR(r_interface);
-    List rcpp_distribution(CAR(r_interface));
-
-    r_interface = CDR(r_interface);
-    List rcpp_searchConfig(CAR(r_interface));
-
-    r_interface = CDR(r_interface);
-    List rcpp_options(CAR(r_interface));
-
-    r_interface = CDR(r_interface);
-    List rcpp_marginalz(CAR(r_interface));
+    // r_interface = CDR(r_interface);
+    // List rcpp_model(CAR(r_interface));
+    // 
+    // r_interface = CDR(r_interface);
+    // List rcpp_data(CAR(r_interface));
+    // 
+    // r_interface = CDR(r_interface);
+    // List rcpp_fpInfos(CAR(r_interface));
+    // 
+    // r_interface = CDR(r_interface);
+    // List rcpp_ucInfos(CAR(r_interface));
+    // 
+    // r_interface = CDR(r_interface);
+    // List rcpp_fixInfos(CAR(r_interface));
+    // 
+    // r_interface = CDR(r_interface);
+    // List rcpp_distribution(CAR(r_interface));
+    // 
+    // r_interface = CDR(r_interface);
+    // List rcpp_searchConfig(CAR(r_interface));
+    // 
+    // r_interface = CDR(r_interface);
+    // List rcpp_options(CAR(r_interface));
+    // 
+    // r_interface = CDR(r_interface);
+    // List rcpp_marginalz(CAR(r_interface));
 
     // ----------------------------------------------------------------------------------
     // unpack the R objects
@@ -373,7 +383,20 @@ cpp_sampleGlm(SEXP r_interface)
     {
         ucColList.push_back(as<PosIntVector>(rcpp_ucColList[i]));
     }
-
+    
+    // fixed covariate configuration:
+    
+    const PosIntVector fixIndices = rcpp_fixInfos["fixIndices"];
+    List rcpp_fixColList = rcpp_fixInfos["fixColList"];
+    
+    std::vector<PosIntVector> fixColList;
+    for (R_len_t i = 0; i != rcpp_fixColList.length(); ++i)
+    {
+      fixColList.push_back(as<PosIntVector>(rcpp_fixColList[i]));
+    }
+    
+    
+    
 
     // distributions info:
 
@@ -385,6 +408,7 @@ cpp_sampleGlm(SEXP r_interface)
     const bool doGlm = as<bool>(rcpp_distribution["doGlm"]);
     
     const double empiricalMean = as<double>(rcpp_distribution["yMean"]);
+    const bool empiricalgPrior = as<bool>(rcpp_distribution["empiricalgPrior"]);
     
     // model search configuration:
     const bool useFixedc = as<bool>(rcpp_searchConfig["useFixedc"]);
@@ -444,9 +468,29 @@ cpp_sampleGlm(SEXP r_interface)
      }
      const UcInfo ucInfo(ucSizes, maxUcDim, ucIndices, ucColList);
 
+  
+     // fix configuration:  
+     
+     // determine sizes of the fix groups, and the total size == maximum size reached together by all
+     // UC groups.
+     PosIntVector fixSizes;
+     PosInt maxFixDim = 0;
+     for (std::vector<PosIntVector>::const_iterator cols = fixColList.begin(); cols != fixColList.end(); ++cols)
+     {
+       PosInt thisSize = cols->size();
+       
+       maxFixDim += thisSize;
+       fixSizes.push_back(thisSize);
+     }
+     const FixInfo fixInfo(fixSizes, maxFixDim, fixIndices, fixColList);
+     
+     
+     
+     
+     
      // model configuration:
      GlmModelConfig config(rcpp_family, nullModelLogMargLik, nullModelDeviance, exp(fixedZ), rcpp_gPrior,
-                           data.response, debug, useFixedc, empiricalMean);
+                           data.response, debug, useFixedc, empiricalMean, empiricalgPrior);
 
 
      // model config/info:
@@ -498,6 +542,7 @@ cpp_sampleGlm(SEXP r_interface)
                                       data,
                                       fpInfo,
                                       ucInfo,
+                                      fixInfo,
                                       config,
                                       config.linPredStart,
                                       options.useFixedZ,
@@ -515,7 +560,7 @@ cpp_sampleGlm(SEXP r_interface)
      }
      else
      {
-         AMatrix design = getDesignMatrix(thisModel.par, data, fpInfo, ucInfo, false);
+         AMatrix design = getDesignMatrix(thisModel.par, data, fpInfo, ucInfo, fixInfo, false);
          fitter.coxfitObject = new Coxfit(data.response,
                                           data.censInd,
                                           design,
@@ -739,8 +784,11 @@ cpp_sampleGlm(SEXP r_interface)
 
                  // compute the shrinkage factor t = g / (g + 1)
                  const double g = exp(now.sample.z);
-                 const double shrinkFactor = g / (g + 1.0);
-
+                
+                //Previously used g directly, but if g=inf we need to use the limit
+                 // const double shrinkFactor = g / (g + 1.0);
+                const double shrinkFactor = std::isinf(g) ? 1 : g / (g + 1.0);
+                 
                  // scale the variance of the non-intercept coefficients
                  // with this factor.
                  // In the Cox case: no intercept present, so scale everything
